/*
*******************************
* 1990 Mustang GT Info Center *
*           v3.6              *
*        May 13, 2013          *
*         Sean Neal           *
*******************************

This program uses a 20x4 LCD I installed in place of my stock
speedometer to display useful information similar to this:

▀▀█ ▀▀█      12:34am
█▀▀ ▀▀█ 12.3v    NNE
█▄▄ ▄▄█ 200°F   86°F
123,456mi Trip:123.4

-  The speed, time, and heading are all updated by the GPS module.
-  The battery voltage is taken directly from the car battery using
   a voltage divider to convert 0-15VDC to 0-5VDC.
-  The engine temperature is calculated from a 0-5VDC input from the
   stock ECT (engine coolant temperature) sensor. This information
   is also used to cycle the electric cooling fan on at 220 deg F and
   off at 210 deg F using a 5V coil relay controlling a heavy-duty
   solenoid under the hood.
-  The outside temperature is taken from a DS18B20 serial temp probe
   mounted at the front of the vehicle behind the bumper cover.
-  The odometer and tripmeter values are stored in the (NV)RAM of an RTC
   module. A momentary pushbutton rest switch is mounted in the stock
   location for the factory mechanical trip meter reset.

************************   
Portions of this program were copied from or inspired by the following:

Big LCD number font:
Hifiduino
http://hifiduino.blogspot.com/

Overall program structure and odometer/tripmeter coding:
Kurt Schulz' Scooterputer
http://www.janspace.com/b2evolution/arduino.php/2010/06/26/scooterputer

Tom Torfs' 8 bit converter macro:
http://bytes.com/topic/c/answers/216333-binary-constant-macros

*************************
Hardware:

Arduino UNO R3
http://www.amazon.com/Arduino-UNO-board-DIP-ATmega328P/dp/B006H06TVG

SainSmart GPS Module
http://www.amazon.com/SainSmart-Aircraft-Controller-Mega2560-Duemilanove/dp/B009YDK6NA

Antenna GPS 3V Magnetic Mount SMA
http://www.sparkfun.com/products/464

Seeed STR104B2P Protoshield
http://www.radioshack.com/product/index.jsp?productId=13297702

SainSmart 4-Channel 5V Relay Module
http://www.amazon.com/SainSmart-4-Channel-Relay-Module-Arduino/dp/B0057OC5O8

Real-time Clock (RTC)
http://www.amazon.com/Real-time-Clock-RTC-for-Arduino/dp/B00ATRD4OS
*/

#include <LiquidCrystal_I2C.h>
#include <LCD.h>
#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <TimedAction.h>
#include <Wire.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <DS1307.h>

// binary constant generator macro
#define HEX__(n) 0x##n##LU

// 8-bit conversion function
#define B8__(x) ((x&0x0000000FLU)?1:0) \
+((x&0x000000F0LU)?2:0) \
+((x&0x00000F00LU)?4:0) \
+((x&0x0000F000LU)?8:0) \
+((x&0x000F0000LU)?16:0) \
+((x&0x00F00000LU)?32:0) \
+((x&0x0F000000LU)?64:0) \
+((x&0xF0000000LU)?128:0)
#define B8(d) ((unsigned char)B8__(HEX__(d)))

// Digital pin assignments
#define PUSHBUTTON_PIN	        2  // Momentary switch mounted in place of old trip reset button
#define GREEN_LED_PIN           13 // onboard green LED
#define GPS_RX_PIN		3  
#define GPS_TX_PIN		4
#define TEMP_PIN		5  // Outside Temperature probe (DS18B20)
#define RELAY1_PIN              11 // Cooling Fan Relay (12v)
#define RELAY2_PIN              10 // Spare Relay (12v)
#define RELAY3_PIN              9  // Spare Relay (12v)
#define RELAY4_PIN              8  // Spare Relay (12v)

// Analog pin assignments
#define BATT_PIN		2  // Battery Voltage (0-5v through voltage divider)
#define ECT_PIN			3  // Engine Coolant Temperature (0-5v)

// RTC non-volatile storage
#define RTC_I2C_ADDR  0x68         // Real time clock address on the I2C bus
#define RAM_ADDR      8            // RTC RAM registers start at address 8
#define TRIP_ADDR     RAM_ADDR     // Trips, ODO, lat and long are all floats, 4 bytes ea.         
#define ODO_ADDR      TRIP_ADDR + 4
#define LAT_ADDR      ODO_ADDR + 4
#define LON_ADDR      LAT_ADDR + 4

#define MILES_PER_METER         0.00062137f
#define EARTH_RADIUS_METERS     6372795.0f

// Tasks - TimedAction( msec, funcName )
TimedAction ECT_UpdateTask = TimedAction( 10000, ECT_Update );      // Update ECT display every 10 seconds
TimedAction BATT_UpdateTask = TimedAction( 2000, BATT_Update );     // Update battery voltage display every 2 seconds
//TimedAction TEMP_UpdateTask = TimedAction( 10000, TEMP_Update );  // Update outside temp display every 10 seconds

// Interfaces
TinyGPS gps;
DS1307 clock;
SoftwareSerial GPS_Serial = SoftwareSerial(GPS_RX_PIN, GPS_TX_PIN);
OneWire oneWire(TEMP_PIN);
DallasTemperature TEMP_Sensor(&oneWire);
LiquidCrystal_I2C lcd(0x3F,2,1,0,4,5,6,7);  // SainSmart 20x4 LCD w/I2C board

// Constants
const char Version[ ] = ("3.6");            // Version
const bool DST = true;                      // Daylight Savings Time (add hardware switch for this later)
const int FAN_ON_TEMP = 200;                // Temp at which cooling fan turns on (should be 210...lower for testing purposes)
const int FAN_OFF_TEMP = FAN_ON_TEMP - 10;  // Temp at which cooling fan turns off (should be 10 less)

// GPS
float latitude = 0.0;
float longitude = 0.0;
float f_Trip = 0.0;
float f_ODO = 0.0;

//Sensor values
int valMPH		= 0;		// Miles per hour, GPS: 0 to 99
long valODO             = 0L;           // Odometer in miles, internal: 0 to 999,999
float valTrip           = 0;            // Tripmeter miles, internal: 0 to 999.9
int valTemp		= 0;		// Outside temperature, TEMP: -255 to 255 (degrees F)
int valECT		= 0;		// ECT temperature, ECT: 32 to 302 (degrees F)
float valBatt		= 0;		// Battery voltage, BATT: 0 to 15.00 volts
int valLatDegrs		= 0;		// Latitude degrees, GPS
long valLatFrac		= 0L;		// Latitude fraction, GPS
int valLonDegrs		= 0;		// Longitude degrees, GPS
long valLonFrac		= 0L;		// Longitude fraction, GPS
bool FAN_ON             = false;        // Fan status

void setup()
{
  pinMode(RELAY1_PIN, OUTPUT);                  // Enable Relay 1
  digitalWrite(RELAY1_PIN, HIGH);               // set to "off"
  pinMode(RELAY2_PIN, OUTPUT);                  // Enable Relay 2
  digitalWrite(RELAY2_PIN, HIGH);               // set to "off"
  pinMode(RELAY3_PIN, OUTPUT);                  // Enable Relay 3
  digitalWrite(RELAY3_PIN, HIGH);               // set to "off"
  pinMode(RELAY4_PIN, OUTPUT);                  // Enable Relay 4
  digitalWrite(RELAY4_PIN, HIGH);               // set to "off"
 
  delay(4000); 					// pause for engine start
    
  lcd.setBacklightPin(3,POSITIVE);		// Switch on the backlight
  lcd.setBacklight(HIGH);
  lcd.begin(20,4);				// Start LCD interface
  DefineLargeChar(); 				// Create and send the custom characters to the LCD

  GPS_Serial.begin(9600);			// Start GPS interface
  clock.begin();                                // Start RTC interface
  
  ECT_UpdateTask.enable( );                     // Enable task for reading the ECT sensor
  BATT_UpdateTask.enable( );                    // Enable task for reading the battery voltage
  //TEMP_Update();				// Get initial temperature reading
  //TEMP_UpdateTask.enable( );                  // Enable task for reading the outside temperature sensor
  
  //  Read trip, ODO, and location data from RTC storage to init vals
  f_Trip = RTC_ReadFloat( TRIP_ADDR );
  f_ODO   = RTC_ReadFloat( ODO_ADDR );
  
  latitude = RTC_ReadFloat( LAT_ADDR );
  longitude = RTC_ReadFloat( LON_ADDR );
            
  valTrip = f_Trip;
  valODO  = f_ODO;
  
  //  KEEP THIS - Needed to set the odometer
  //  f_ODO = 128000.0f;
  //  valODO = f_ODO;
  //  RTC_WriteFloat( ODO_ADDR, f_ODO );
   
  //  KEEP THIS - Needed to set the time and date
  //  clock.fillByYMD(2013,5,3);                  // Set Date
  //  clock.fillByHMS(2,43,00);                   // Set Time (24HR)
  //  clock.fillDayOfWeek(FRI);                   // Set day of week
  //  clock.setTime();                            // write time to the RTC chip
  
  //  TEMP_Sensor.begin();		          // Start Temperature interface
  
  pinMode(PUSHBUTTON_PIN, INPUT);	          // Enable Pushbutton for Trip reset
  pinMode(GREEN_LED_PIN, OUTPUT);                 // Enable onboard green LED
  
  Splash_Screen();
  LCD_Template();   
}

//*********************
// Main Loop
//
void loop()
{ 
  GPS_Update( );
  ECT_UpdateTask.check( );
  BATT_UpdateTask.check( );
  //TEMP_UpdateTask.check( );
    
  //Reset Trip meter
  if (digitalRead(PUSHBUTTON_PIN) == HIGH){
    f_Trip = 0;
    lcd.setCursor(15,3);
    lcd.print("000.0");
    RTC_WriteFloat( TRIP_ADDR, f_Trip );}      
}

//**************************
// Splash Screen
//
void Splash_Screen()
{
  lcd.clear();
  lcd.setCursor(2, 1);
  lcd.print("1990 Mustang GT");
  lcd.setCursor(2, 2);
  lcd.print("Info Center v");lcd.print(Version);
  delay(3000);
}

//**************************
// LCD Template
//
void LCD_Template()
{
  bool PM_temp = false;
  byte hour_temp;  
  
  // Clear the display and show NO GPS SIG until GPS is active
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("NO");
  lcd.setCursor(0,1);
  lcd.print("GPS");
  lcd.setCursor(0,2);
  lcd.print("SIG");
  
  // Display static time once from RTC...this gets replaced with GPS data once GPS becomes active
  lcd.setCursor(13,0);
  clock.getTime();
  
  // Change to 12 hour format am/pm
  if (clock.hour >= 12){
    PM_temp = true;}
  if (clock.hour > 12){
    hour_temp = clock.hour - 12;}
  if (clock.hour == 0){
    hour_temp = 12;}
    
  if (hour_temp < 10){
    lcd.print(" ");}
  lcd.print(hour_temp, DEC);lcd.print(":");
  if (clock.minute < 10){
    lcd.print("0");}
  lcd.print(clock.minute, DEC);
  if (PM_temp == true){
    lcd.print("pm");}
    else {
      lcd.print("am");}
      
  // Battery voltage and heading
  lcd.setCursor(8,1);
  lcd.print("--.-v    ---");
  
  // ECT and outside temp
  lcd.setCursor(8,2);
  lcd.print("---"); lcd.print(char(223)); lcd.print("F  ---");lcd.print(char(223)); lcd.print("F");
  lcd.setCursor(7,3);
  lcd.print("mi Trip:");
  
  // Odometer and Tripmeter from RTC NVRAM
  display_odo_trip( );
}

/*
//*********************
// Temperature DS18B20
//
void TEMP_Update()
{
  TEMP_Sensor.requestTemperature();
  valTemp = (int)TEMP_Sensor.getTempFByIndex(0);
}
*/

//*********************
// Battery Monitor
//
void BATT_Update()
{
  valBatt = analogRead(BATT_PIN);
  valBatt = map(valBatt,0,1023,0,1585);  // Mapping the 0-5v input to 0-15v battery voltage
  valBatt = valBatt / 10;
  valBatt = valBatt + 0.5;
  valBatt = int(valBatt);
  valBatt = valBatt / 10;
  lcd.setCursor(8,1);
  if (valBatt < 10){
    lcd.print(" ");}
  lcd.print(valBatt,1);
}

//*********************
// ECT Monitor
//
void ECT_Update()
{
  valECT = analogRead(ECT_PIN);
  valECT = map(valECT,60,750,200,0);                // Mapping the 0-5V input to deg F
  
  // Fan Control
  if (valECT >= FAN_ON_TEMP && FAN_ON == false){    // ECT >= 210 and fan NOT running,
    digitalWrite(RELAY1_PIN, LOW);                  // start fan
    FAN_ON = true;                }                 
  if (valECT <= FAN_OFF_TEMP && FAN_ON == true){    // ECT <= 200 and fan IS running,
    digitalWrite(RELAY1_PIN, HIGH);                 // stop fan
    FAN_ON = false;}
    
  // Write ECT to LCD  
  lcd.setCursor(8,2);
  if (valECT < 100){
    lcd.print("LOW");}                              // display "LOW" if temp under 100 deg F
    else {
      lcd.print(valECT);}
  lcd.setCursor(13,2);
  if (FAN_ON == true){                              // display FAN ON indicator dot next to temp
    lcd.print(char(165));}
    else {
      lcd.print(" ");}
}

//*****************************
// Display ODO and Trip on LCD
//
int ODO_thousands, ODO_hundreds;

void display_odo_trip()
{
  lcd.setCursor(0,3);
  ODO_thousands = int( f_ODO / 1000 );
  ODO_hundreds = f_ODO - ( ODO_thousands * 1000 );
  if (ODO_thousands < 100 && ODO_thousands >= 10){
  lcd.print("0");}
  else if (ODO_thousands < 10){
    lcd.print("00");}
    lcd.print(ODO_thousands);lcd.print(",");
    if (ODO_hundreds < 100 && ODO_hundreds >= 10){
      lcd.print("0");}
      else if (ODO_hundreds < 10){
        lcd.print("00");}
    lcd.print(ODO_hundreds);
    lcd.setCursor(15,3);
    if (f_Trip < 100 && f_Trip >=10){
      lcd.print("0");}
      else if (f_Trip < 10){
        lcd.print("00");}
    lcd.print(f_Trip,1);
}

//************************
// GPS Update and display 
// SPEED, TIME, HEADING
//

  long lat, lon;
  float f_lat, f_lon;
  unsigned long age = 0;
  int speed = 0;
  float mph = 0.0;
  float distance = 0.0;
  unsigned long date, time, chars;
  int year;
  byte month, day, hour, minute, second, hundredths;
  
void GPS_Update()  
{
  while(GPS_Serial.available())
  {
    if(gps.encode(GPS_Serial.read()))            // Returns true if valid NMEA sentence received
    {
      digitalWrite(GREEN_LED_PIN, HIGH); 	 // turn on green LED when GPS is active
      gps.get_position(&lat, &lon, &age);
      gps.f_get_position(&f_lat, &f_lon, &age);
      
      // Speed display
      mph = gps.f_speed_mph();
      speed = int( mph + 0.5 );
      if ( speed < 10 )
      {
        lcd.setCursor(0,0);lcd.print("   ");    // Blank 1st digit if speed < 10mph
        lcd.setCursor(0,1);lcd.print("   ");
        lcd.setCursor(0,2);lcd.print("   ");
        if ( speed < 2 ){
          printOneNumber(0);}                   // Ignore speeds of less than 2 mph and display 0 instead
          else {
            printOneNumber(speed);}             // Print single digit speed
      }
        else if ( speed > 9 && speed < 100 )
        {
          printTwoNumber(speed);                // Print double digit speed
        }
          else if ( speed > 99 )
          {
            printTwoNumber(99);                 // 99 is max speed displayed due to LCD space constraints
          }
    
    // If we're moving, calculate and save distance traveled
    if( mph > 2.0f )
    {
      if( latitude != 0.0f )
      {
        float distance = DistanceBetween2Points( latitude, longitude, f_lat, f_lon, MILES_PER_METER );
        
        if( distance >= 0.05f )      // Accumulate every 20th of a mile
        {
          f_Trip += distance;
          f_ODO += distance;
          
          if( f_Trip > 999.9f )     // Reset trip meter after 999.9 miles
            f_Trip = 0.0f;
          display_odo_trip( );      // Write to LCD   
          latitude = f_lat;
          longitude = f_lon;
          
          // Save trips and ODO floats to RTC storage
          RTC_WriteFloat( TRIP_ADDR, f_Trip );
          RTC_WriteFloat( ODO_ADDR, f_ODO );
        }
      }
        else
        {
          latitude = f_lat;
          longitude = f_lon;
        }
      // Save latitude and longitude to RTC storage
      RTC_WriteFloat( LAT_ADDR, latitude );
      RTC_WriteFloat( LON_ADDR, longitude );
    }
 
    // Time display
    int TZ = 5;                              // Eastern Standard Time difference to GMT (my equations only work for Western hemisphere)
    if (DST == true){
      TZ = TZ - 1;}
    bool PM = false;
    lcd.setCursor(13,0);
    gps.crack_datetime(&year,&month,&day,&hour,&minute,&second,&hundredths);
    if (hour < TZ){
      hour = hour + 24 - TZ;}
      else if (hour >= TZ){
        hour = hour - TZ;}
    
    // Set RTC time from GPS only if hours or minutes don't match
    if (clock.hour != hour || clock.minute != minute){
      clock.fillByHMS(hour,minute,second);
      clock.setTime();}
    
    // Change to 12-hour am/pm
    if (hour >= 12){
      PM = true;}
    if (hour > 12){
      hour = hour - 12;}
    if (hour == 0){
      hour = 12;}
      
    if (hour<10){
      lcd.print(" ");}
    lcd.print(hour, DEC); 
    if (second % 2){                          // flash the ":" between hour and minute every second while GPS active
      lcd.print(":");}  
      else {lcd.print(" ");}
    if (minute < 10){lcd.print("0");}
    lcd.print(minute, DEC);
    if (PM == false){
      lcd.print("a");}
      else {
        lcd.print("p");}
  
    // Heading display
    lcd.setCursor(17,1);
    lcd.print(gps.cardinal(gps.f_course()));  // cardinal() in TinyGPS.cpp has been modified to right-justify
                                              // the headings to make it work with my display scheme
    }
  }
}

//************************
// Great Circle distance calculation
// Returns the distance between two lat/lon points on Earth
// (Note: Assumes level traveling, does not take into account changes in altitude...close enough!)
//
float DistanceBetween2Points( float Lat1, float Lon1, float Lat2, float Lon2, float unit_conversion)
{
  float dLat = radians( Lat2 - Lat1 );
  float dLon = radians( Lon2 - Lon1 );
  
  float a = sin( dLat / 2.0f) * sin(dLat / 2.0f) +
            cos( radians( Lat1 ) ) * cos( radians( Lat2 ) ) *
            sin( dLon / 2.0f ) * sin( dLon / 2.0f );
  float d = 2.0f * atan2( sqrt( a ), sqrt( 1.0f - a) );
  return d * EARTH_RADIUS_METERS * unit_conversion;
}

//************************
// Custom LCD characters
//
void DefineLargeChar()
{
  // A 1 in the binary representation of the character means it is filled in
  // characters are 5 pixels wide by 8 pixels tall

  // We need 7 custom characters for the display.
 
  // Define Custom Characters

  uint8_t cc1[8] = {     // Custom Character 1
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111)
  };

  uint8_t cc2[8] = {    // Custom Character 2
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(00000),
    B8(00000),
    B8(00000)
  };

  uint8_t cc3[8] = {    // Custom Character 3
    B8(00000),
    B8(00000),
    B8(00000),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111)
  };

  uint8_t cc4[8] = {   // Custom Character 4
    B8(00000),
    B8(00000),
    B8(00000),
    B8(00000),
    B8(00000),
    B8(00000),
    B8(00000),
    B8(00000)
  };

  uint8_t cc5[8] = {    // Custom Character 5
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(00000),
    B8(00000)
  };

  uint8_t cc6[8] = {    // Custom Character 6
    B8(00000),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(00000),
    B8(00000)
  };

  uint8_t cc7[8] = {     // Custom Character 7
    B8(00000),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111),
    B8(11111)
  };

  // send custom characters to the display
  lcd.createChar(1,cc1);
  lcd.createChar(2,cc2);
  lcd.createChar(3,cc3);
  lcd.createChar(4,cc4);
  lcd.createChar(5,cc5);
  lcd.createChar(6,cc6);
  lcd.createChar(7,cc7);

}

// Array index into parts of big numbers. Numbers consist of 9 custom characters in 3 lines
//            0      1      2      3      4      5      6      7      8      9    
char bn1[]={1,2,1, 3,1,4, 2,2,1, 2,2,1, 1,4,1, 1,2,2, 1,2,2, 2,2,1, 1,2,1, 1,2,1};
char bn2[]={1,4,1, 4,1,4, 7,6,5, 6,6,1, 5,6,1, 5,6,7, 1,6,7, 4,4,1, 1,6,1, 5,6,1};
char bn3[]={1,3,1, 3,1,3, 1,3,3, 3,3,1, 4,4,1, 3,3,1, 1,3,1, 4,4,1, 1,3,1, 3,3,1};

void printOneNumber(uint8_t digit)
{
  // Print position is hardcoded

  // Line 1 of the one digit number
  lcd.setCursor(4,0);
  lcd.write(bn1[digit*3]);
  lcd.write(bn1[digit*3+1]);
  lcd.write(bn1[digit*3+2]);

  // Line 2 of the one-digit number
  lcd.setCursor(4,1);
  lcd.write(bn2[digit*3]);
  lcd.write(bn2[digit*3+1]);
  lcd.write(bn2[digit*3+2]);

  // Line 3 of the one-digit number
  lcd.setCursor(4,2);
  lcd.write(bn3[digit*3]);
  lcd.write(bn3[digit*3+1]);
  lcd.write(bn3[digit*3+2]);
}

void printTwoNumber(uint8_t number)
{
  // Print position is hardcoded
  int digit0;  // To represent the ones
  int digit1;  // To represent the tens
  digit0=number%10;
  digit1=number/10;

  // Line 1 of the two-digit number
  lcd.setCursor(0,0);
  lcd.write(bn1[digit1*3]);
  lcd.write(bn1[digit1*3+1]);
  lcd.write(bn1[digit1*3+2]);
  lcd.write(0x04); // Blank
  lcd.write(bn1[digit0*3]);
  lcd.write(bn1[digit0*3+1]);
  lcd.write(bn1[digit0*3+2]);

  // Line 2 of the two-digit number
  lcd.setCursor(0,1);
  lcd.write(bn2[digit1*3]);
  lcd.write(bn2[digit1*3+1]);
  lcd.write(bn2[digit1*3+2]);
  lcd.write(0x04); // Blank
  lcd.write(bn2[digit0*3]);
  lcd.write(bn2[digit0*3+1]);
  lcd.write(bn2[digit0*3+2]);

  // Line 3 of the two-digit number
  lcd.setCursor(0,2);
  lcd.write(bn3[digit1*3]);
  lcd.write(bn3[digit1*3+1]);
  lcd.write(bn3[digit1*3+2]);
  lcd.write(0x04); // Blank
  lcd.write(bn3[digit0*3]);
  lcd.write(bn3[digit0*3+1]);
  lcd.write(bn3[digit0*3+2]);
}

//**************************
// RTC DS1307
// Utility functions

// Convert decimal to BCD
byte decToBcd( byte val )
{
  return ( ( val / 10 * 16) + ( val % 10 ) );
}

// Convert BCD to decimal
byte bcdToDec( byte val )
{
  return ( ( val / 16 * 10) + ( val % 16 ) );
}

//**************************
// RTC DS1307
// RAM register access functions
//

// Reads a 2-byte integer value from the RTC RAM registers
int RTC_ReadInteger( int valAddr )
{
  // Set the register pointer
  Wire.beginTransmission( RTC_I2C_ADDR );
  Wire.write( valAddr );
  Wire.endTransmission( );

  // Read 2 bytes into int value
  Wire.requestFrom( RTC_I2C_ADDR, 2 );
  int value = Wire.read( );
  value = ( value << 8 ) + Wire.read( );
  
  return value;
}

// Reads a 4-byte float value from the RTC RAM registers
float RTC_ReadFloat( int valAddr )
{
  float value;
  byte  *byteArray = (byte *) &value;
  
  // Set the register pointer
  Wire.beginTransmission( RTC_I2C_ADDR );
  Wire.write( valAddr );
  Wire.endTransmission( );

  // Read 4 bytes can convert to float value
  Wire.requestFrom( RTC_I2C_ADDR, 4 );
  byteArray[3] = Wire.read( );
  byteArray[2] = Wire.read( );
  byteArray[1] = Wire.read( );
  byteArray[0] = Wire.read( );
  
  return value;
}

// Writes a 2-byte integer value to the RTC RAM registers
void RTC_WriteInteger( int valAddr, int value )
{
  if( valAddr > 7 && valAddr  < 63 )  // Don't let writes go to the RTC registers 0 - 7
  {
    Wire.beginTransmission( RTC_I2C_ADDR );
    Wire.write( valAddr );

    // Write high byte, low byte
    Wire.write( ( unsigned char )( value >> 8 ) );
    Wire.write( ( unsigned char )value ); 
    
    Wire.endTransmission( );
  }  
}

// Writes a 4-byte float value to the RTC RAM registers
void RTC_WriteFloat( int valAddr, float value )
{
  if( valAddr > 7 && valAddr  < 61 )  // Don't let writes go to the RTC registers 0 - 7
  {
    Wire.beginTransmission( RTC_I2C_ADDR );
    Wire.write( valAddr );
    
    // Write high word (high byte/low byte), low word (high byte/low byte)
    byte  *byteArray;
    byteArray = (byte *) &value;
    Wire.write( byteArray[3] );
    Wire.write( byteArray[2] );
    Wire.write( byteArray[1] );
    Wire.write( byteArray[0] );

    Wire.endTransmission( );
  } 
}

